sqlglot.dialects.presto API documentation

Edit on GitHub

# sqlglot.dialects.presto

View Source

      1from __future__ import annotations
      2
      3import typing as t
      4
      5from sqlglot import exp, generator, parser, tokens, transforms
      6from sqlglot.dialects.dialect import (
      7    Dialect,
      8    date_trunc_to_time,
      9    format_time_lambda,
     10    if_sql,
     11    left_to_substring_sql,
     12    no_ilike_sql,
     13    no_pivot_sql,
     14    no_safe_divide_sql,
     15    rename_func,
     16    right_to_substring_sql,
     17    struct_extract_sql,
     18    timestamptrunc_sql,
     19    timestrtotime_sql,
     20)
     21from sqlglot.dialects.mysql import MySQL
     22from sqlglot.errors import UnsupportedError
     23from sqlglot.helper import apply_index_offset, seq_get
     24from sqlglot.tokens import TokenType
     25
     26
     27def _approx_distinct_sql(self: generator.Generator, expression: exp.ApproxDistinct) -> str:
     28    accuracy = expression.args.get("accuracy")
     29    accuracy = ", " + self.sql(accuracy) if accuracy else ""
     30    return f"APPROX_DISTINCT({self.sql(expression, 'this')}{accuracy})"
     31
     32
     33def _datatype_sql(self: generator.Generator, expression: exp.DataType) -> str:
     34    sql = self.datatype_sql(expression)
     35    if expression.is_type("timestamptz"):
     36        sql = f"{sql} WITH TIME ZONE"
     37    return sql
     38
     39
     40def _explode_to_unnest_sql(self: generator.Generator, expression: exp.Lateral) -> str:
     41    if isinstance(expression.this, (exp.Explode, exp.Posexplode)):
     42        return self.sql(
     43            exp.Join(
     44                this=exp.Unnest(
     45                    expressions=[expression.this.this],
     46                    alias=expression.args.get("alias"),
     47                    ordinality=isinstance(expression.this, exp.Posexplode),
     48                ),
     49                kind="cross",
     50            )
     51        )
     52    return self.lateral_sql(expression)
     53
     54
     55def _initcap_sql(self: generator.Generator, expression: exp.Initcap) -> str:
     56    regex = r"(\w)(\w*)"
     57    return f"REGEXP_REPLACE({self.sql(expression, 'this')}, '{regex}', x -> UPPER(x[1]) || LOWER(x[2]))"
     58
     59
     60def _decode_sql(self: generator.Generator, expression: exp.Decode) -> str:
     61    _ensure_utf8(expression.args["charset"])
     62    return self.func("FROM_UTF8", expression.this, expression.args.get("replace"))
     63
     64
     65def _encode_sql(self: generator.Generator, expression: exp.Encode) -> str:
     66    _ensure_utf8(expression.args["charset"])
     67    return f"TO_UTF8({self.sql(expression, 'this')})"
     68
     69
     70def _no_sort_array(self: generator.Generator, expression: exp.SortArray) -> str:
     71    if expression.args.get("asc") == exp.false():
     72        comparator = "(a, b) -> CASE WHEN a < b THEN 1 WHEN a > b THEN -1 ELSE 0 END"
     73    else:
     74        comparator = None
     75    return self.func("ARRAY_SORT", expression.this, comparator)
     76
     77
     78def _schema_sql(self: generator.Generator, expression: exp.Schema) -> str:
     79    if isinstance(expression.parent, exp.Property):
     80        columns = ", ".join(f"'{c.name}'" for c in expression.expressions)
     81        return f"ARRAY[{columns}]"
     82
     83    if expression.parent:
     84        for schema in expression.parent.find_all(exp.Schema):
     85            if isinstance(schema.parent, exp.Property):
     86                expression = expression.copy()
     87                expression.expressions.extend(schema.expressions)
     88
     89    return self.schema_sql(expression)
     90
     91
     92def _quantile_sql(self: generator.Generator, expression: exp.Quantile) -> str:
     93    self.unsupported("Presto does not support exact quantiles")
     94    return f"APPROX_PERCENTILE({self.sql(expression, 'this')}, {self.sql(expression, 'quantile')})"
     95
     96
     97def _str_to_time_sql(
     98    self: generator.Generator, expression: exp.StrToDate | exp.StrToTime | exp.TsOrDsToDate
     99) -> str:
    100    return f"DATE_PARSE({self.sql(expression, 'this')}, {self.format_time(expression)})"
    101
    102
    103def _ts_or_ds_to_date_sql(self: generator.Generator, expression: exp.TsOrDsToDate) -> str:
    104    time_format = self.format_time(expression)
    105    if time_format and time_format not in (Presto.TIME_FORMAT, Presto.DATE_FORMAT):
    106        return exp.cast(_str_to_time_sql(self, expression), "DATE").sql(dialect="presto")
    107    return exp.cast(exp.cast(expression.this, "TIMESTAMP"), "DATE").sql(dialect="presto")
    108
    109
    110def _ts_or_ds_add_sql(self: generator.Generator, expression: exp.TsOrDsAdd) -> str:
    111    this = expression.this
    112
    113    if not isinstance(this, exp.CurrentDate):
    114        this = exp.cast(exp.cast(expression.this, "TIMESTAMP"), "DATE")
    115
    116    return self.func(
    117        "DATE_ADD",
    118        exp.Literal.string(expression.text("unit") or "day"),
    119        expression.expression,
    120        this,
    121    )
    122
    123
    124def _ensure_utf8(charset: exp.Literal) -> None:
    125    if charset.name.lower() != "utf-8":
    126        raise UnsupportedError(f"Unsupported charset {charset}")
    127
    128
    129def _approx_percentile(args: t.List) -> exp.Expression:
    130    if len(args) == 4:
    131        return exp.ApproxQuantile(
    132            this=seq_get(args, 0),
    133            weight=seq_get(args, 1),
    134            quantile=seq_get(args, 2),
    135            accuracy=seq_get(args, 3),
    136        )
    137    if len(args) == 3:
    138        return exp.ApproxQuantile(
    139            this=seq_get(args, 0), quantile=seq_get(args, 1), accuracy=seq_get(args, 2)
    140        )
    141    return exp.ApproxQuantile.from_arg_list(args)
    142
    143
    144def _from_unixtime(args: t.List) -> exp.Expression:
    145    if len(args) == 3:
    146        return exp.UnixToTime(
    147            this=seq_get(args, 0),
    148            hours=seq_get(args, 1),
    149            minutes=seq_get(args, 2),
    150        )
    151    if len(args) == 2:
    152        return exp.UnixToTime(this=seq_get(args, 0), zone=seq_get(args, 1))
    153
    154    return exp.UnixToTime.from_arg_list(args)
    155
    156
    157def _parse_element_at(args: t.List) -> exp.SafeBracket:
    158    this = seq_get(args, 0)
    159    index = seq_get(args, 1)
    160    assert isinstance(this, exp.Expression) and isinstance(index, exp.Expression)
    161    return exp.SafeBracket(this=this, expressions=apply_index_offset(this, [index], -1))
    162
    163
    164def _unnest_sequence(expression: exp.Expression) -> exp.Expression:
    165    if isinstance(expression, exp.Table):
    166        if isinstance(expression.this, exp.GenerateSeries):
    167            unnest = exp.Unnest(expressions=[expression.this])
    168
    169            if expression.alias:
    170                return exp.alias_(unnest, alias="_u", table=[expression.alias], copy=False)
    171            return unnest
    172    return expression
    173
    174
    175class Presto(Dialect):
    176    INDEX_OFFSET = 1
    177    NULL_ORDERING = "nulls_are_last"
    178    TIME_FORMAT = MySQL.TIME_FORMAT
    179    TIME_MAPPING = MySQL.TIME_MAPPING
    180    STRICT_STRING_CONCAT = True
    181
    182    # https://github.com/trinodb/trino/issues/17
    183    # https://github.com/trinodb/trino/issues/12289
    184    # https://github.com/prestodb/presto/issues/2863
    185    RESOLVES_IDENTIFIERS_AS_UPPERCASE = None
    186
    187    class Tokenizer(tokens.Tokenizer):
    188        KEYWORDS = {
    189            **tokens.Tokenizer.KEYWORDS,
    190            "START": TokenType.BEGIN,
    191            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    192            "ROW": TokenType.STRUCT,
    193        }
    194
    195    class Parser(parser.Parser):
    196        FUNCTIONS = {
    197            **parser.Parser.FUNCTIONS,
    198            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    199            "APPROX_PERCENTILE": _approx_percentile,
    200            "CARDINALITY": exp.ArraySize.from_arg_list,
    201            "CONTAINS": exp.ArrayContains.from_arg_list,
    202            "DATE_ADD": lambda args: exp.DateAdd(
    203                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    204            ),
    205            "DATE_DIFF": lambda args: exp.DateDiff(
    206                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    207            ),
    208            "DATE_FORMAT": format_time_lambda(exp.TimeToStr, "presto"),
    209            "DATE_PARSE": format_time_lambda(exp.StrToTime, "presto"),
    210            "DATE_TRUNC": date_trunc_to_time,
    211            "ELEMENT_AT": _parse_element_at,
    212            "FROM_HEX": exp.Unhex.from_arg_list,
    213            "FROM_UNIXTIME": _from_unixtime,
    214            "FROM_UTF8": lambda args: exp.Decode(
    215                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    216            ),
    217            "NOW": exp.CurrentTimestamp.from_arg_list,
    218            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    219            "STRPOS": lambda args: exp.StrPosition(
    220                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    221            ),
    222            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    223            "TO_HEX": exp.Hex.from_arg_list,
    224            "TO_UTF8": lambda args: exp.Encode(
    225                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    226            ),
    227        }
    228        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    229        FUNCTION_PARSERS.pop("TRIM")
    230
    231    class Generator(generator.Generator):
    232        INTERVAL_ALLOWS_PLURAL_FORM = False
    233        JOIN_HINTS = False
    234        TABLE_HINTS = False
    235        IS_BOOL_ALLOWED = False
    236        STRUCT_DELIMITER = ("(", ")")
    237
    238        PROPERTIES_LOCATION = {
    239            **generator.Generator.PROPERTIES_LOCATION,
    240            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    241            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    242        }
    243
    244        TYPE_MAPPING = {
    245            **generator.Generator.TYPE_MAPPING,
    246            exp.DataType.Type.INT: "INTEGER",
    247            exp.DataType.Type.FLOAT: "REAL",
    248            exp.DataType.Type.BINARY: "VARBINARY",
    249            exp.DataType.Type.TEXT: "VARCHAR",
    250            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    251            exp.DataType.Type.STRUCT: "ROW",
    252        }
    253
    254        TRANSFORMS = {
    255            **generator.Generator.TRANSFORMS,
    256            exp.ApproxDistinct: _approx_distinct_sql,
    257            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    258            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    259            exp.ArrayConcat: rename_func("CONCAT"),
    260            exp.ArrayContains: rename_func("CONTAINS"),
    261            exp.ArraySize: rename_func("CARDINALITY"),
    262            exp.BitwiseAnd: lambda self, e: f"BITWISE_AND({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    263            exp.BitwiseLeftShift: lambda self, e: f"BITWISE_ARITHMETIC_SHIFT_LEFT({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    264            exp.BitwiseNot: lambda self, e: f"BITWISE_NOT({self.sql(e, 'this')})",
    265            exp.BitwiseOr: lambda self, e: f"BITWISE_OR({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    266            exp.BitwiseRightShift: lambda self, e: f"BITWISE_ARITHMETIC_SHIFT_RIGHT({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    267            exp.BitwiseXor: lambda self, e: f"BITWISE_XOR({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    268            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    269            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    270            exp.DataType: _datatype_sql,
    271            exp.DateAdd: lambda self, e: self.func(
    272                "DATE_ADD", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
    273            ),
    274            exp.DateDiff: lambda self, e: self.func(
    275                "DATE_DIFF", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
    276            ),
    277            exp.DateStrToDate: lambda self, e: f"CAST(DATE_PARSE({self.sql(e, 'this')}, {Presto.DATE_FORMAT}) AS DATE)",
    278            exp.DateToDi: lambda self, e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    279            exp.Decode: _decode_sql,
    280            exp.DiToDate: lambda self, e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    281            exp.Encode: _encode_sql,
    282            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    283            exp.Group: transforms.preprocess([transforms.unalias_group]),
    284            exp.Hex: rename_func("TO_HEX"),
    285            exp.If: if_sql,
    286            exp.ILike: no_ilike_sql,
    287            exp.Initcap: _initcap_sql,
    288            exp.Lateral: _explode_to_unnest_sql,
    289            exp.Left: left_to_substring_sql,
    290            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    291            exp.LogicalAnd: rename_func("BOOL_AND"),
    292            exp.LogicalOr: rename_func("BOOL_OR"),
    293            exp.Pivot: no_pivot_sql,
    294            exp.Quantile: _quantile_sql,
    295            exp.Right: right_to_substring_sql,
    296            exp.SafeBracket: lambda self, e: self.func(
    297                "ELEMENT_AT", e.this, seq_get(apply_index_offset(e.this, e.expressions, 1), 0)
    298            ),
    299            exp.SafeDivide: no_safe_divide_sql,
    300            exp.Schema: _schema_sql,
    301            exp.Select: transforms.preprocess(
    302                [
    303                    transforms.eliminate_qualify,
    304                    transforms.eliminate_distinct_on,
    305                    transforms.explode_to_unnest,
    306                ]
    307            ),
    308            exp.SortArray: _no_sort_array,
    309            exp.StrPosition: rename_func("STRPOS"),
    310            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    311            exp.StrToTime: _str_to_time_sql,
    312            exp.StrToUnix: lambda self, e: f"TO_UNIXTIME(DATE_PARSE({self.sql(e, 'this')}, {self.format_time(e)}))",
    313            exp.StructExtract: struct_extract_sql,
    314            exp.Table: transforms.preprocess([_unnest_sequence]),
    315            exp.TimestampTrunc: timestamptrunc_sql,
    316            exp.TimeStrToDate: timestrtotime_sql,
    317            exp.TimeStrToTime: timestrtotime_sql,
    318            exp.TimeStrToUnix: lambda self, e: f"TO_UNIXTIME(DATE_PARSE({self.sql(e, 'this')}, {Presto.TIME_FORMAT}))",
    319            exp.TimeToStr: lambda self, e: f"DATE_FORMAT({self.sql(e, 'this')}, {self.format_time(e)})",
    320            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    321            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    322            exp.TsOrDiToDi: lambda self, e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    323            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    324            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    325            exp.Unhex: rename_func("FROM_HEX"),
    326            exp.UnixToStr: lambda self, e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    327            exp.UnixToTime: rename_func("FROM_UNIXTIME"),
    328            exp.UnixToTimeStr: lambda self, e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    329            exp.VariancePop: rename_func("VAR_POP"),
    330            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    331            exp.WithinGroup: transforms.preprocess(
    332                [transforms.remove_within_group_for_percentiles]
    333            ),
    334        }
    335
    336        def interval_sql(self, expression: exp.Interval) -> str:
    337            unit = self.sql(expression, "unit")
    338            if expression.this and unit.lower().startswith("week"):
    339                return f"({expression.this.name} * INTERVAL '7' day)"
    340            return super().interval_sql(expression)
    341
    342        def transaction_sql(self, expression: exp.Transaction) -> str:
    343            modes = expression.args.get("modes")
    344            modes = f" {', '.join(modes)}" if modes else ""
    345            return f"START TRANSACTION{modes}"
    346
    347        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    348            start = expression.args["start"]
    349            end = expression.args["end"]
    350            step = expression.args.get("step")
    351
    352            if isinstance(start, exp.Cast):
    353                target_type = start.to
    354            elif isinstance(end, exp.Cast):
    355                target_type = end.to
    356            else:
    357                target_type = None
    358
    359            if target_type and target_type.is_type("timestamp"):
    360                to = target_type.copy()
    361
    362                if target_type is start.to:
    363                    end = exp.cast(end, to)
    364                else:
    365                    start = exp.cast(start, to)
    366
    367            return self.func("SEQUENCE", start, end, step)
    368
    369        def offset_limit_modifiers(
    370            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    371        ) -> t.List[str]:
    372            return [
    373                self.sql(expression, "offset"),
    374                self.sql(limit),
    375            ]

class Presto(sqlglot.dialects.dialect.Dialect): View Source

    176class Presto(Dialect):
    177    INDEX_OFFSET = 1
    178    NULL_ORDERING = "nulls_are_last"
    179    TIME_FORMAT = MySQL.TIME_FORMAT
    180    TIME_MAPPING = MySQL.TIME_MAPPING
    181    STRICT_STRING_CONCAT = True
    182
    183    # https://github.com/trinodb/trino/issues/17
    184    # https://github.com/trinodb/trino/issues/12289
    185    # https://github.com/prestodb/presto/issues/2863
    186    RESOLVES_IDENTIFIERS_AS_UPPERCASE = None
    187
    188    class Tokenizer(tokens.Tokenizer):
    189        KEYWORDS = {
    190            **tokens.Tokenizer.KEYWORDS,
    191            "START": TokenType.BEGIN,
    192            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    193            "ROW": TokenType.STRUCT,
    194        }
    195
    196    class Parser(parser.Parser):
    197        FUNCTIONS = {
    198            **parser.Parser.FUNCTIONS,
    199            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    200            "APPROX_PERCENTILE": _approx_percentile,
    201            "CARDINALITY": exp.ArraySize.from_arg_list,
    202            "CONTAINS": exp.ArrayContains.from_arg_list,
    203            "DATE_ADD": lambda args: exp.DateAdd(
    204                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    205            ),
    206            "DATE_DIFF": lambda args: exp.DateDiff(
    207                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    208            ),
    209            "DATE_FORMAT": format_time_lambda(exp.TimeToStr, "presto"),
    210            "DATE_PARSE": format_time_lambda(exp.StrToTime, "presto"),
    211            "DATE_TRUNC": date_trunc_to_time,
    212            "ELEMENT_AT": _parse_element_at,
    213            "FROM_HEX": exp.Unhex.from_arg_list,
    214            "FROM_UNIXTIME": _from_unixtime,
    215            "FROM_UTF8": lambda args: exp.Decode(
    216                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    217            ),
    218            "NOW": exp.CurrentTimestamp.from_arg_list,
    219            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    220            "STRPOS": lambda args: exp.StrPosition(
    221                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    222            ),
    223            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    224            "TO_HEX": exp.Hex.from_arg_list,
    225            "TO_UTF8": lambda args: exp.Encode(
    226                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    227            ),
    228        }
    229        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    230        FUNCTION_PARSERS.pop("TRIM")
    231
    232    class Generator(generator.Generator):
    233        INTERVAL_ALLOWS_PLURAL_FORM = False
    234        JOIN_HINTS = False
    235        TABLE_HINTS = False
    236        IS_BOOL_ALLOWED = False
    237        STRUCT_DELIMITER = ("(", ")")
    238
    239        PROPERTIES_LOCATION = {
    240            **generator.Generator.PROPERTIES_LOCATION,
    241            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    242            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    243        }
    244
    245        TYPE_MAPPING = {
    246            **generator.Generator.TYPE_MAPPING,
    247            exp.DataType.Type.INT: "INTEGER",
    248            exp.DataType.Type.FLOAT: "REAL",
    249            exp.DataType.Type.BINARY: "VARBINARY",
    250            exp.DataType.Type.TEXT: "VARCHAR",
    251            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    252            exp.DataType.Type.STRUCT: "ROW",
    253        }
    254
    255        TRANSFORMS = {
    256            **generator.Generator.TRANSFORMS,
    257            exp.ApproxDistinct: _approx_distinct_sql,
    258            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    259            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    260            exp.ArrayConcat: rename_func("CONCAT"),
    261            exp.ArrayContains: rename_func("CONTAINS"),
    262            exp.ArraySize: rename_func("CARDINALITY"),
    263            exp.BitwiseAnd: lambda self, e: f"BITWISE_AND({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    264            exp.BitwiseLeftShift: lambda self, e: f"BITWISE_ARITHMETIC_SHIFT_LEFT({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    265            exp.BitwiseNot: lambda self, e: f"BITWISE_NOT({self.sql(e, 'this')})",
    266            exp.BitwiseOr: lambda self, e: f"BITWISE_OR({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    267            exp.BitwiseRightShift: lambda self, e: f"BITWISE_ARITHMETIC_SHIFT_RIGHT({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    268            exp.BitwiseXor: lambda self, e: f"BITWISE_XOR({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    269            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    270            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    271            exp.DataType: _datatype_sql,
    272            exp.DateAdd: lambda self, e: self.func(
    273                "DATE_ADD", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
    274            ),
    275            exp.DateDiff: lambda self, e: self.func(
    276                "DATE_DIFF", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
    277            ),
    278            exp.DateStrToDate: lambda self, e: f"CAST(DATE_PARSE({self.sql(e, 'this')}, {Presto.DATE_FORMAT}) AS DATE)",
    279            exp.DateToDi: lambda self, e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    280            exp.Decode: _decode_sql,
    281            exp.DiToDate: lambda self, e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    282            exp.Encode: _encode_sql,
    283            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    284            exp.Group: transforms.preprocess([transforms.unalias_group]),
    285            exp.Hex: rename_func("TO_HEX"),
    286            exp.If: if_sql,
    287            exp.ILike: no_ilike_sql,
    288            exp.Initcap: _initcap_sql,
    289            exp.Lateral: _explode_to_unnest_sql,
    290            exp.Left: left_to_substring_sql,
    291            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    292            exp.LogicalAnd: rename_func("BOOL_AND"),
    293            exp.LogicalOr: rename_func("BOOL_OR"),
    294            exp.Pivot: no_pivot_sql,
    295            exp.Quantile: _quantile_sql,
    296            exp.Right: right_to_substring_sql,
    297            exp.SafeBracket: lambda self, e: self.func(
    298                "ELEMENT_AT", e.this, seq_get(apply_index_offset(e.this, e.expressions, 1), 0)
    299            ),
    300            exp.SafeDivide: no_safe_divide_sql,
    301            exp.Schema: _schema_sql,
    302            exp.Select: transforms.preprocess(
    303                [
    304                    transforms.eliminate_qualify,
    305                    transforms.eliminate_distinct_on,
    306                    transforms.explode_to_unnest,
    307                ]
    308            ),
    309            exp.SortArray: _no_sort_array,
    310            exp.StrPosition: rename_func("STRPOS"),
    311            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    312            exp.StrToTime: _str_to_time_sql,
    313            exp.StrToUnix: lambda self, e: f"TO_UNIXTIME(DATE_PARSE({self.sql(e, 'this')}, {self.format_time(e)}))",
    314            exp.StructExtract: struct_extract_sql,
    315            exp.Table: transforms.preprocess([_unnest_sequence]),
    316            exp.TimestampTrunc: timestamptrunc_sql,
    317            exp.TimeStrToDate: timestrtotime_sql,
    318            exp.TimeStrToTime: timestrtotime_sql,
    319            exp.TimeStrToUnix: lambda self, e: f"TO_UNIXTIME(DATE_PARSE({self.sql(e, 'this')}, {Presto.TIME_FORMAT}))",
    320            exp.TimeToStr: lambda self, e: f"DATE_FORMAT({self.sql(e, 'this')}, {self.format_time(e)})",
    321            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    322            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    323            exp.TsOrDiToDi: lambda self, e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    324            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    325            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    326            exp.Unhex: rename_func("FROM_HEX"),
    327            exp.UnixToStr: lambda self, e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    328            exp.UnixToTime: rename_func("FROM_UNIXTIME"),
    329            exp.UnixToTimeStr: lambda self, e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    330            exp.VariancePop: rename_func("VAR_POP"),
    331            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    332            exp.WithinGroup: transforms.preprocess(
    333                [transforms.remove_within_group_for_percentiles]
    334            ),
    335        }
    336
    337        def interval_sql(self, expression: exp.Interval) -> str:
    338            unit = self.sql(expression, "unit")
    339            if expression.this and unit.lower().startswith("week"):
    340                return f"({expression.this.name} * INTERVAL '7' day)"
    341            return super().interval_sql(expression)
    342
    343        def transaction_sql(self, expression: exp.Transaction) -> str:
    344            modes = expression.args.get("modes")
    345            modes = f" {', '.join(modes)}" if modes else ""
    346            return f"START TRANSACTION{modes}"
    347
    348        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    349            start = expression.args["start"]
    350            end = expression.args["end"]
    351            step = expression.args.get("step")
    352
    353            if isinstance(start, exp.Cast):
    354                target_type = start.to
    355            elif isinstance(end, exp.Cast):
    356                target_type = end.to
    357            else:
    358                target_type = None
    359
    360            if target_type and target_type.is_type("timestamp"):
    361                to = target_type.copy()
    362
    363                if target_type is start.to:
    364                    end = exp.cast(end, to)
    365                else:
    366                    start = exp.cast(start, to)
    367
    368            return self.func("SEQUENCE", start, end, step)
    369
    370        def offset_limit_modifiers(
    371            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    372        ) -> t.List[str]:
    373            return [
    374                self.sql(expression, "offset"),
    375                self.sql(limit),
    376            ]

INDEX_OFFSET = 1
NULL_ORDERING = 'nulls_are_last'
TIME_FORMAT = "'%Y-%m-%d %T'"
TIME_MAPPING: Dict[str, str] = {'%M': '%B', '%c': '%-m', '%e': '%-d', '%h': '%I', '%i': '%M', '%s': '%S', '%S': '%S', '%u': '%W', '%k': '%-H', '%l': '%-I', '%T': '%H:%M:%S', '%W': '%a'}
STRICT_STRING_CONCAT = True
RESOLVES_IDENTIFIERS_AS_UPPERCASE: Optional[bool] = None
tokenizer_class = <class 'sqlglot.dialects.presto.Presto.Tokenizer'>
parser_class = <class 'sqlglot.dialects.presto.Presto.Parser'>
generator_class = <class 'sqlglot.dialects.presto.Presto.Generator'>
TIME_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'S': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
FORMAT_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'S': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
INVERSE_TIME_MAPPING: Dict[str, str] = {'%B': '%M', '%-m': '%c', '%-d': '%e', '%I': '%h', '%M': '%i', '%S': '%S', '%W': '%u', '%-H': '%k', '%-I': '%l', '%H:%M:%S': '%T', '%a': '%W'}
INVERSE_TIME_TRIE: Dict = {'%': {'B': {0: True}, '-': {'m': {0: True}, 'd': {0: True}, 'H': {0: True}, 'I': {0: True}}, 'I': {0: True}, 'M': {0: True}, 'S': {0: True}, 'W': {0: True}, 'H': {':': {'%': {'M': {':': {'%': {'S': {0: True}}}}}}}, 'a': {0: True}}}
QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
BIT_START = None
BIT_END = None
HEX_START = None
HEX_END = None
BYTE_START = None
BYTE_END = None
RAW_START = None
RAW_END = None

##### Inherited Members

sqlglot.dialects.dialect.Dialect
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  NORMALIZE_FUNCTIONS
  DATE_FORMAT
  DATEINT_FORMAT
  FORMAT_MAPPING
  get_or_raise
  format_time
  normalize_identifier
  case_sensitive
  can_identify
  quote_identifier
  parse
  parse_into
  generate
  transpile
  tokenize
  tokenizer
  parser
  generator

class Presto.Tokenizer(sqlglot.tokens.Tokenizer): View Source

    188    class Tokenizer(tokens.Tokenizer):
    189        KEYWORDS = {
    190            **tokens.Tokenizer.KEYWORDS,
    191            "START": TokenType.BEGIN,
    192            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    193            "ROW": TokenType.STRUCT,
    194        }

KEYWORDS = {'{%': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%-': <TokenType.BLOCK_START: 'BLOCK_START'>, '%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '+%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '{{+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{{-': <TokenType.BLOCK_START: 'BLOCK_START'>, '+}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '/*+': <TokenType.HINT: 'HINT'>, '==': <TokenType.EQ: 'EQ'>, '::': <TokenType.DCOLON: 'DCOLON'>, '||': <TokenType.DPIPE: 'DPIPE'>, '>=': <TokenType.GTE: 'GTE'>, '<=': <TokenType.LTE: 'LTE'>, '<>': <TokenType.NEQ: 'NEQ'>, '!=': <TokenType.NEQ: 'NEQ'>, '<=>': <TokenType.NULLSAFE_EQ: 'NULLSAFE_EQ'>, '->': <TokenType.ARROW: 'ARROW'>, '->>': <TokenType.DARROW: 'DARROW'>, '=>': <TokenType.FARROW: 'FARROW'>, '#>': <TokenType.HASH_ARROW: 'HASH_ARROW'>, '#>>': <TokenType.DHASH_ARROW: 'DHASH_ARROW'>, '<->': <TokenType.LR_ARROW: 'LR_ARROW'>, '&&': <TokenType.DAMP: 'DAMP'>, 'ALL': <TokenType.ALL: 'ALL'>, 'ALWAYS': <TokenType.ALWAYS: 'ALWAYS'>, 'AND': <TokenType.AND: 'AND'>, 'ANTI': <TokenType.ANTI: 'ANTI'>, 'ANY': <TokenType.ANY: 'ANY'>, 'ASC': <TokenType.ASC: 'ASC'>, 'AS': <TokenType.ALIAS: 'ALIAS'>, 'ASOF': <TokenType.ASOF: 'ASOF'>, 'AUTOINCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'AUTO_INCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'BEGIN': <TokenType.BEGIN: 'BEGIN'>, 'BETWEEN': <TokenType.BETWEEN: 'BETWEEN'>, 'CACHE': <TokenType.CACHE: 'CACHE'>, 'UNCACHE': <TokenType.UNCACHE: 'UNCACHE'>, 'CASE': <TokenType.CASE: 'CASE'>, 'CHARACTER SET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'CLUSTER BY': <TokenType.CLUSTER_BY: 'CLUSTER_BY'>, 'COLLATE': <TokenType.COLLATE: 'COLLATE'>, 'COLUMN': <TokenType.COLUMN: 'COLUMN'>, 'COMMIT': <TokenType.COMMIT: 'COMMIT'>, 'CONSTRAINT': <TokenType.CONSTRAINT: 'CONSTRAINT'>, 'CREATE': <TokenType.CREATE: 'CREATE'>, 'CROSS': <TokenType.CROSS: 'CROSS'>, 'CUBE': <TokenType.CUBE: 'CUBE'>, 'CURRENT_DATE': <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, 'CURRENT_TIME': <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, 'CURRENT_TIMESTAMP': <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, 'CURRENT_USER': <TokenType.CURRENT_USER: 'CURRENT_USER'>, 'DATABASE': <TokenType.DATABASE: 'DATABASE'>, 'DEFAULT': <TokenType.DEFAULT: 'DEFAULT'>, 'DELETE': <TokenType.DELETE: 'DELETE'>, 'DESC': <TokenType.DESC: 'DESC'>, 'DESCRIBE': <TokenType.DESCRIBE: 'DESCRIBE'>, 'DISTINCT': <TokenType.DISTINCT: 'DISTINCT'>, 'DISTRIBUTE BY': <TokenType.DISTRIBUTE_BY: 'DISTRIBUTE_BY'>, 'DIV': <TokenType.DIV: 'DIV'>, 'DROP': <TokenType.DROP: 'DROP'>, 'ELSE': <TokenType.ELSE: 'ELSE'>, 'END': <TokenType.END: 'END'>, 'ESCAPE': <TokenType.ESCAPE: 'ESCAPE'>, 'EXCEPT': <TokenType.EXCEPT: 'EXCEPT'>, 'EXECUTE': <TokenType.EXECUTE: 'EXECUTE'>, 'EXISTS': <TokenType.EXISTS: 'EXISTS'>, 'FALSE': <TokenType.FALSE: 'FALSE'>, 'FETCH': <TokenType.FETCH: 'FETCH'>, 'FILTER': <TokenType.FILTER: 'FILTER'>, 'FIRST': <TokenType.FIRST: 'FIRST'>, 'FULL': <TokenType.FULL: 'FULL'>, 'FUNCTION': <TokenType.FUNCTION: 'FUNCTION'>, 'FOR': <TokenType.FOR: 'FOR'>, 'FOREIGN KEY': <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, 'FORMAT': <TokenType.FORMAT: 'FORMAT'>, 'FROM': <TokenType.FROM: 'FROM'>, 'GEOGRAPHY': <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, 'GEOMETRY': <TokenType.GEOMETRY: 'GEOMETRY'>, 'GLOB': <TokenType.GLOB: 'GLOB'>, 'GROUP BY': <TokenType.GROUP_BY: 'GROUP_BY'>, 'GROUPING SETS': <TokenType.GROUPING_SETS: 'GROUPING_SETS'>, 'HAVING': <TokenType.HAVING: 'HAVING'>, 'IF': <TokenType.IF: 'IF'>, 'ILIKE': <TokenType.ILIKE: 'ILIKE'>, 'IN': <TokenType.IN: 'IN'>, 'INDEX': <TokenType.INDEX: 'INDEX'>, 'INET': <TokenType.INET: 'INET'>, 'INNER': <TokenType.INNER: 'INNER'>, 'INSERT': <TokenType.INSERT: 'INSERT'>, 'INTERVAL': <TokenType.INTERVAL: 'INTERVAL'>, 'INTERSECT': <TokenType.INTERSECT: 'INTERSECT'>, 'INTO': <TokenType.INTO: 'INTO'>, 'IS': <TokenType.IS: 'IS'>, 'ISNULL': <TokenType.ISNULL: 'ISNULL'>, 'JOIN': <TokenType.JOIN: 'JOIN'>, 'KEEP': <TokenType.KEEP: 'KEEP'>, 'LATERAL': <TokenType.LATERAL: 'LATERAL'>, 'LEFT': <TokenType.LEFT: 'LEFT'>, 'LIKE': <TokenType.LIKE: 'LIKE'>, 'LIMIT': <TokenType.LIMIT: 'LIMIT'>, 'LOAD': <TokenType.LOAD: 'LOAD'>, 'LOCK': <TokenType.LOCK: 'LOCK'>, 'MERGE': <TokenType.MERGE: 'MERGE'>, 'NATURAL': <TokenType.NATURAL: 'NATURAL'>, 'NEXT': <TokenType.NEXT: 'NEXT'>, 'NEXT VALUE FOR': <TokenType.NEXT_VALUE_FOR: 'NEXT_VALUE_FOR'>, 'NOT': <TokenType.NOT: 'NOT'>, 'NOTNULL': <TokenType.NOTNULL: 'NOTNULL'>, 'NULL': <TokenType.NULL: 'NULL'>, 'OBJECT': <TokenType.OBJECT: 'OBJECT'>, 'OFFSET': <TokenType.OFFSET: 'OFFSET'>, 'ON': <TokenType.ON: 'ON'>, 'OR': <TokenType.OR: 'OR'>, 'ORDER BY': <TokenType.ORDER_BY: 'ORDER_BY'>, 'ORDINALITY': <TokenType.ORDINALITY: 'ORDINALITY'>, 'OUTER': <TokenType.OUTER: 'OUTER'>, 'OVER': <TokenType.OVER: 'OVER'>, 'OVERLAPS': <TokenType.OVERLAPS: 'OVERLAPS'>, 'OVERWRITE': <TokenType.OVERWRITE: 'OVERWRITE'>, 'PARTITION': <TokenType.PARTITION: 'PARTITION'>, 'PARTITION BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED_BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PERCENT': <TokenType.PERCENT: 'PERCENT'>, 'PIVOT': <TokenType.PIVOT: 'PIVOT'>, 'PRAGMA': <TokenType.PRAGMA: 'PRAGMA'>, 'PRIMARY KEY': <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, 'PROCEDURE': <TokenType.PROCEDURE: 'PROCEDURE'>, 'QUALIFY': <TokenType.QUALIFY: 'QUALIFY'>, 'RANGE': <TokenType.RANGE: 'RANGE'>, 'RECURSIVE': <TokenType.RECURSIVE: 'RECURSIVE'>, 'REGEXP': <TokenType.RLIKE: 'RLIKE'>, 'REPLACE': <TokenType.REPLACE: 'REPLACE'>, 'RETURNING': <TokenType.RETURNING: 'RETURNING'>, 'REFERENCES': <TokenType.REFERENCES: 'REFERENCES'>, 'RIGHT': <TokenType.RIGHT: 'RIGHT'>, 'RLIKE': <TokenType.RLIKE: 'RLIKE'>, 'ROLLBACK': <TokenType.ROLLBACK: 'ROLLBACK'>, 'ROLLUP': <TokenType.ROLLUP: 'ROLLUP'>, 'ROW': <TokenType.STRUCT: 'STRUCT'>, 'ROWS': <TokenType.ROWS: 'ROWS'>, 'SCHEMA': <TokenType.SCHEMA: 'SCHEMA'>, 'SELECT': <TokenType.SELECT: 'SELECT'>, 'SEMI': <TokenType.SEMI: 'SEMI'>, 'SET': <TokenType.SET: 'SET'>, 'SETTINGS': <TokenType.SETTINGS: 'SETTINGS'>, 'SHOW': <TokenType.SHOW: 'SHOW'>, 'SIMILAR TO': <TokenType.SIMILAR_TO: 'SIMILAR_TO'>, 'SOME': <TokenType.SOME: 'SOME'>, 'SORT BY': <TokenType.SORT_BY: 'SORT_BY'>, 'TABLE': <TokenType.TABLE: 'TABLE'>, 'TABLESAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TEMP': <TokenType.TEMPORARY: 'TEMPORARY'>, 'TEMPORARY': <TokenType.TEMPORARY: 'TEMPORARY'>, 'THEN': <TokenType.THEN: 'THEN'>, 'TRUE': <TokenType.TRUE: 'TRUE'>, 'UNION': <TokenType.UNION: 'UNION'>, 'UNNEST': <TokenType.UNNEST: 'UNNEST'>, 'UNPIVOT': <TokenType.UNPIVOT: 'UNPIVOT'>, 'UPDATE': <TokenType.UPDATE: 'UPDATE'>, 'USE': <TokenType.USE: 'USE'>, 'USING': <TokenType.USING: 'USING'>, 'UUID': <TokenType.UUID: 'UUID'>, 'VALUES': <TokenType.VALUES: 'VALUES'>, 'VIEW': <TokenType.VIEW: 'VIEW'>, 'VOLATILE': <TokenType.VOLATILE: 'VOLATILE'>, 'WHEN': <TokenType.WHEN: 'WHEN'>, 'WHERE': <TokenType.WHERE: 'WHERE'>, 'WINDOW': <TokenType.WINDOW: 'WINDOW'>, 'WITH': <TokenType.WITH: 'WITH'>, 'APPLY': <TokenType.APPLY: 'APPLY'>, 'ARRAY': <TokenType.ARRAY: 'ARRAY'>, 'BIT': <TokenType.BIT: 'BIT'>, 'BOOL': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BOOLEAN': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BYTE': <TokenType.TINYINT: 'TINYINT'>, 'TINYINT': <TokenType.TINYINT: 'TINYINT'>, 'SHORT': <TokenType.SMALLINT: 'SMALLINT'>, 'SMALLINT': <TokenType.SMALLINT: 'SMALLINT'>, 'INT2': <TokenType.SMALLINT: 'SMALLINT'>, 'INTEGER': <TokenType.INT: 'INT'>, 'INT': <TokenType.INT: 'INT'>, 'INT4': <TokenType.INT: 'INT'>, 'LONG': <TokenType.BIGINT: 'BIGINT'>, 'BIGINT': <TokenType.BIGINT: 'BIGINT'>, 'INT8': <TokenType.BIGINT: 'BIGINT'>, 'DEC': <TokenType.DECIMAL: 'DECIMAL'>, 'DECIMAL': <TokenType.DECIMAL: 'DECIMAL'>, 'BIGDECIMAL': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'BIGNUMERIC': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'MAP': <TokenType.MAP: 'MAP'>, 'NULLABLE': <TokenType.NULLABLE: 'NULLABLE'>, 'NUMBER': <TokenType.DECIMAL: 'DECIMAL'>, 'NUMERIC': <TokenType.DECIMAL: 'DECIMAL'>, 'FIXED': <TokenType.DECIMAL: 'DECIMAL'>, 'REAL': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT4': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT8': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE PRECISION': <TokenType.DOUBLE: 'DOUBLE'>, 'JSON': <TokenType.JSON: 'JSON'>, 'CHAR': <TokenType.CHAR: 'CHAR'>, 'CHARACTER': <TokenType.CHAR: 'CHAR'>, 'NCHAR': <TokenType.NCHAR: 'NCHAR'>, 'VARCHAR': <TokenType.VARCHAR: 'VARCHAR'>, 'VARCHAR2': <TokenType.VARCHAR: 'VARCHAR'>, 'NVARCHAR': <TokenType.NVARCHAR: 'NVARCHAR'>, 'NVARCHAR2': <TokenType.NVARCHAR: 'NVARCHAR'>, 'STR': <TokenType.TEXT: 'TEXT'>, 'STRING': <TokenType.TEXT: 'TEXT'>, 'TEXT': <TokenType.TEXT: 'TEXT'>, 'CLOB': <TokenType.TEXT: 'TEXT'>, 'LONGVARCHAR': <TokenType.TEXT: 'TEXT'>, 'BINARY': <TokenType.BINARY: 'BINARY'>, 'BLOB': <TokenType.VARBINARY: 'VARBINARY'>, 'BYTEA': <TokenType.VARBINARY: 'VARBINARY'>, 'VARBINARY': <TokenType.VARBINARY: 'VARBINARY'>, 'TIME': <TokenType.TIME: 'TIME'>, 'TIMESTAMP': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMPTZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPLTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'DATE': <TokenType.DATE: 'DATE'>, 'DATETIME': <TokenType.DATETIME: 'DATETIME'>, 'INT4RANGE': <TokenType.INT4RANGE: 'INT4RANGE'>, 'INT4MULTIRANGE': <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, 'INT8RANGE': <TokenType.INT8RANGE: 'INT8RANGE'>, 'INT8MULTIRANGE': <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, 'NUMRANGE': <TokenType.NUMRANGE: 'NUMRANGE'>, 'NUMMULTIRANGE': <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, 'TSRANGE': <TokenType.TSRANGE: 'TSRANGE'>, 'TSMULTIRANGE': <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, 'TSTZRANGE': <TokenType.TSTZRANGE: 'TSTZRANGE'>, 'TSTZMULTIRANGE': <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, 'DATERANGE': <TokenType.DATERANGE: 'DATERANGE'>, 'DATEMULTIRANGE': <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, 'UNIQUE': <TokenType.UNIQUE: 'UNIQUE'>, 'STRUCT': <TokenType.STRUCT: 'STRUCT'>, 'VARIANT': <TokenType.VARIANT: 'VARIANT'>, 'ALTER': <TokenType.ALTER: 'ALTER'>, 'ANALYZE': <TokenType.COMMAND: 'COMMAND'>, 'CALL': <TokenType.COMMAND: 'COMMAND'>, 'COMMENT': <TokenType.COMMENT: 'COMMENT'>, 'COPY': <TokenType.COMMAND: 'COMMAND'>, 'EXPLAIN': <TokenType.COMMAND: 'COMMAND'>, 'GRANT': <TokenType.COMMAND: 'COMMAND'>, 'OPTIMIZE': <TokenType.COMMAND: 'COMMAND'>, 'PREPARE': <TokenType.COMMAND: 'COMMAND'>, 'TRUNCATE': <TokenType.COMMAND: 'COMMAND'>, 'VACUUM': <TokenType.COMMAND: 'COMMAND'>, 'USER-DEFINED': <TokenType.USERDEFINED: 'USERDEFINED'>, 'START': <TokenType.BEGIN: 'BEGIN'>, 'MATCH_RECOGNIZE': <TokenType.MATCH_RECOGNIZE: 'MATCH_RECOGNIZE'>}

##### Inherited Members

sqlglot.tokens.Tokenizer
  SINGLE_TOKENS
  BIT_STRINGS
  BYTE_STRINGS
  HEX_STRINGS
  RAW_STRINGS
  IDENTIFIERS
  IDENTIFIER_ESCAPES
  QUOTES
  STRING_ESCAPES
  VAR_SINGLE_TOKENS
  IDENTIFIERS_CAN_START_WITH_DIGIT
  WHITE_SPACE
  COMMANDS
  COMMAND_PREFIX_TOKENS
  NUMERIC_LITERALS
  ENCODE
  COMMENTS
  reset
  tokenize
  peek
  size
  sql
  tokens

class Presto.Parser(sqlglot.parser.Parser): View Source

    196    class Parser(parser.Parser):
    197        FUNCTIONS = {
    198            **parser.Parser.FUNCTIONS,
    199            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    200            "APPROX_PERCENTILE": _approx_percentile,
    201            "CARDINALITY": exp.ArraySize.from_arg_list,
    202            "CONTAINS": exp.ArrayContains.from_arg_list,
    203            "DATE_ADD": lambda args: exp.DateAdd(
    204                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    205            ),
    206            "DATE_DIFF": lambda args: exp.DateDiff(
    207                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    208            ),
    209            "DATE_FORMAT": format_time_lambda(exp.TimeToStr, "presto"),
    210            "DATE_PARSE": format_time_lambda(exp.StrToTime, "presto"),
    211            "DATE_TRUNC": date_trunc_to_time,
    212            "ELEMENT_AT": _parse_element_at,
    213            "FROM_HEX": exp.Unhex.from_arg_list,
    214            "FROM_UNIXTIME": _from_unixtime,
    215            "FROM_UTF8": lambda args: exp.Decode(
    216                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    217            ),
    218            "NOW": exp.CurrentTimestamp.from_arg_list,
    219            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    220            "STRPOS": lambda args: exp.StrPosition(
    221                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    222            ),
    223            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    224            "TO_HEX": exp.Hex.from_arg_list,
    225            "TO_UTF8": lambda args: exp.Encode(
    226                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    227            ),
    228        }
    229        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    230        FUNCTION_PARSERS.pop("TRIM")

Parser consumes a list of tokens produced by the Tokenizer and produces a parsed syntax tree.

###### Arguments:

- error_level: The desired error level. Default: ErrorLevel.IMMEDIATE
- error_message_context: Determines the amount of context to capture from a query string when displaying the error message (in number of characters). Default: 100
- max_errors: Maximum number of error messages to include in a raised ParseError. This is only relevant if error_level is ErrorLevel.RAISE. Default: 3

FUNCTIONS = {'ABS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Abs'>>, 'ANY_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_COUNT_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxQuantile'>>, 'ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAll'>>, 'ARRAY_ANY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAny'>>, 'ARRAY_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_JOIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayJoin'>>, 'ARRAY_SIZE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_SORT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySort'>>, 'ARRAY_SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySum'>>, 'ARRAY_UNION_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUnionAgg'>>, 'AVG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Avg'>>, 'CASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Case'>>, 'CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cast'>>, 'CAST_TO_STR_TYPE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CastToStrType'>>, 'CEIL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CEILING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'COALESCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'IFNULL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'NVL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Concat'>>, 'CONCAT_WS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ConcatWs'>>, 'COUNT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Count'>>, 'COUNT_IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'CURRENT_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDate'>>, 'CURRENT_DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDatetime'>>, 'CURRENT_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTime'>>, 'CURRENT_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'CURRENT_USER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentUser'>>, 'DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Date'>>, 'DATE_ADD': <function Presto.Parser.<lambda>>, 'DATEDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_DIFF': <function Presto.Parser.<lambda>>, 'DATEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATE_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateStrToDate'>>, 'DATE_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateSub'>>, 'DATE_TO_DATE_STR': <function Parser.<lambda>>, 'DATE_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateToDi'>>, 'DATE_TRUNC': <function date_trunc_to_time>, 'DATETIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeAdd'>>, 'DATETIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeDiff'>>, 'DATETIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeSub'>>, 'DATETIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeTrunc'>>, 'DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Day'>>, 'DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAYOFMONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAY_OF_WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAYOFWEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAY_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DAYOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DECODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Decode'>>, 'DI_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DiToDate'>>, 'ENCODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Encode'>>, 'EXP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Exp'>>, 'EXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Extract'>>, 'FLOOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Floor'>>, 'FROM_BASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase'>>, 'FROM_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase64'>>, 'GENERATE_SERIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'GREATEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Greatest'>>, 'GROUP_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GroupConcat'>>, 'HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hex'>>, 'HLL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hll'>>, 'IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'INITCAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Initcap'>>, 'JSONB_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtract'>>, 'JSONB_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtractScalar'>>, 'JSON_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONExtract'>>, 'JSON_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONExtractScalar'>>, 'JSON_FORMAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONFormat'>>, 'J_S_O_N_OBJECT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObject'>>, 'LAST_DATE_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDateOfMonth'>>, 'LEAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Least'>>, 'LEFT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Left'>>, 'LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEVENSHTEIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Levenshtein'>>, 'LN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ln'>>, 'LOG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log'>>, 'LOG10': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log10'>>, 'LOG2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log2'>>, 'LOGICAL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOLAND_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'LOGICAL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOLOR_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'LOWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'LCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'MD5': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5'>>, 'MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Map'>>, 'MAP_FROM_ENTRIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MapFromEntries'>>, 'MATCH_AGAINST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MatchAgainst'>>, 'MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Max'>>, 'MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Min'>>, 'MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Month'>>, 'NEXT_VALUE_FOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NextValueFor'>>, 'NUMBER_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NumberToStr'>>, 'NVL2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nvl2'>>, 'OPEN_J_S_O_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.OpenJSON'>>, 'PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParameterizedAgg'>>, 'PERCENTILE_CONT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileCont'>>, 'PERCENTILE_DISC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileDisc'>>, 'POSEXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Posexplode'>>, 'POWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'POW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quantile'>>, 'RANGE_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RangeN'>>, 'READ_CSV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ReadCSV'>>, 'REDUCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Reduce'>>, 'REGEXP_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpExtract'>>, 'REGEXP_I_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpILike'>>, 'REGEXP_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'REGEXP_SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpSplit'>>, 'REPEAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Repeat'>>, 'RIGHT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Right'>>, 'ROUND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Round'>>, 'ROW_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RowNumber'>>, 'SHA': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA1': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA2'>>, 'SAFE_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeConcat'>>, 'SAFE_DIVIDE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeDivide'>>, 'SET_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SetAgg'>>, 'SORT_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SortArray'>>, 'SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Split'>>, 'SQRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sqrt'>>, 'STANDARD_HASH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StandardHash'>>, 'STAR_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StarMap'>>, 'STDDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDDEV_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevPop'>>, 'STDDEV_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevSamp'>>, 'STR_POSITION': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrPosition'>>, 'STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToDate'>>, 'STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToTime'>>, 'STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToUnix'>>, 'STRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'STRUCT_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StructExtract'>>, 'SUBSTRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Substring'>>, 'SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sum'>>, 'TIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeAdd'>>, 'TIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeDiff'>>, 'TIME_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToDate'>>, 'TIME_STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToTime'>>, 'TIME_STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToUnix'>>, 'TIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeSub'>>, 'TIME_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToStr'>>, 'TIME_TO_TIME_STR': <function Parser.<lambda>>, 'TIME_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeTrunc'>>, 'TIMESTAMP_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampAdd'>>, 'TIMESTAMP_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampSub'>>, 'TIMESTAMP_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampTrunc'>>, 'TO_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToBase64'>>, 'TO_CHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToChar'>>, 'TRIM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Trim'>>, 'TRY_CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TryCast'>>, 'TS_OR_DI_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDiToDi'>>, 'TS_OR_DS_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsAdd'>>, 'TS_OR_DS_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToDate'>>, 'TS_OR_DS_TO_DATE_STR': <function Parser.<lambda>>, 'UNHEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'UNIX_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToStr'>>, 'UNIX_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTime'>>, 'UNIX_TO_TIME_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTimeStr'>>, 'UPPER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'UCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'VAR_MAP': <function parse_var_map>, 'VARIANCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'VAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Week'>>, 'WEEK_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WEEKOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WHEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.When'>>, 'X_M_L_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.XMLTable'>>, 'YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Year'>>, 'GLOB': <function Parser.<lambda>>, 'LIKE': <function parse_like>, 'APPROX_PERCENTILE': <function _approx_percentile>, 'CARDINALITY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'DATE_FORMAT': <function format_time_lambda.<locals>._format_time>, 'DATE_PARSE': <function format_time_lambda.<locals>._format_time>, 'ELEMENT_AT': <function _parse_element_at>, 'FROM_HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'FROM_UNIXTIME': <function _from_unixtime>, 'FROM_UTF8': <function Presto.Parser.<lambda>>, 'NOW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'SEQUENCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'STRPOS': <function Presto.Parser.<lambda>>, 'TO_UNIXTIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TO_HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hex'>>, 'TO_UTF8': <function Presto.Parser.<lambda>>}
FUNCTION_PARSERS = {'CAST': <function Parser.<lambda>>, 'CONCAT': <function Parser.<lambda>>, 'CONVERT': <function Parser.<lambda>>, 'DECODE': <function Parser.<lambda>>, 'EXTRACT': <function Parser.<lambda>>, 'JSON_OBJECT': <function Parser.<lambda>>, 'LOG': <function Parser.<lambda>>, 'MATCH': <function Parser.<lambda>>, 'OPENJSON': <function Parser.<lambda>>, 'POSITION': <function Parser.<lambda>>, 'SAFE_CAST': <function Parser.<lambda>>, 'STRING_AGG': <function Parser.<lambda>>, 'SUBSTRING': <function Parser.<lambda>>, 'TRY_CAST': <function Parser.<lambda>>, 'TRY_CONVERT': <function Parser.<lambda>>}
INDEX_OFFSET: int = 1
STRICT_STRING_CONCAT = True
NULL_ORDERING: str = 'nulls_are_last'
SHOW_TRIE: Dict = {}
SET_TRIE: Dict = {'GLOBAL': {0: True}, 'LOCAL': {0: True}, 'SESSION': {0: True}, 'TRANSACTION': {0: True}}
FORMAT_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'S': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
TIME_MAPPING: Dict[str, str] = {'%M': '%B', '%c': '%-m', '%e': '%-d', '%h': '%I', '%i': '%M', '%s': '%S', '%S': '%S', '%u': '%W', '%k': '%-H', '%l': '%-I', '%T': '%H:%M:%S', '%W': '%a'}
TIME_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'S': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}

##### Inherited Members

sqlglot.parser.Parser
  Parser
  NO_PAREN_FUNCTIONS
  NESTED_TYPE_TOKENS
  ENUM_TYPE_TOKENS
  TYPE_TOKENS
  SUBQUERY_PREDICATES
  RESERVED_KEYWORDS
  DB_CREATABLES
  CREATABLES
  ID_VAR_TOKENS
  INTERVAL_VARS
  TABLE_ALIAS_TOKENS
  COMMENT_TABLE_ALIAS_TOKENS
  UPDATE_ALIAS_TOKENS
  TRIM_TYPES
  FUNC_TOKENS
  CONJUNCTION
  EQUALITY
  COMPARISON
  BITWISE
  TERM
  FACTOR
  TIMESTAMPS
  SET_OPERATIONS
  JOIN_METHODS
  JOIN_SIDES
  JOIN_KINDS
  JOIN_HINTS
  LAMBDAS
  COLUMN_OPERATORS
  EXPRESSION_PARSERS
  STATEMENT_PARSERS
  UNARY_PARSERS
  PRIMARY_PARSERS
  PLACEHOLDER_PARSERS
  RANGE_PARSERS
  PROPERTY_PARSERS
  CONSTRAINT_PARSERS
  ALTER_PARSERS
  SCHEMA_UNNAMED_CONSTRAINTS
  NO_PAREN_FUNCTION_PARSERS
  FUNCTIONS_WITH_ALIASED_ARGS
  QUERY_MODIFIER_PARSERS
  SET_PARSERS
  SHOW_PARSERS
  TYPE_LITERAL_PARSERS
  MODIFIABLES
  DDL_SELECT_TOKENS
  PRE_VOLATILE_TOKENS
  TRANSACTION_KIND
  TRANSACTION_CHARACTERISTICS
  INSERT_ALTERNATIVES
  CLONE_KINDS
  TABLE_INDEX_HINT_TOKENS
  WINDOW_ALIAS_TOKENS
  WINDOW_BEFORE_PAREN_TOKENS
  WINDOW_SIDES
  ADD_CONSTRAINT_TOKENS
  STRICT_CAST
  CONCAT_NULL_OUTPUTS_STRING
  PREFIXED_PIVOT_COLUMNS
  IDENTIFY_PIVOT_STRINGS
  LOG_BASE_FIRST
  LOG_DEFAULTS_TO_LN
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  FORMAT_MAPPING
  error_level
  error_message_context
  max_errors
  reset
  parse
  parse_into
  check_errors
  raise_error
  expression
  validate_expression
  errors
  sql

class Presto.Generator(sqlglot.generator.Generator): View Source

    232    class Generator(generator.Generator):
    233        INTERVAL_ALLOWS_PLURAL_FORM = False
    234        JOIN_HINTS = False
    235        TABLE_HINTS = False
    236        IS_BOOL_ALLOWED = False
    237        STRUCT_DELIMITER = ("(", ")")
    238
    239        PROPERTIES_LOCATION = {
    240            **generator.Generator.PROPERTIES_LOCATION,
    241            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    242            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    243        }
    244
    245        TYPE_MAPPING = {
    246            **generator.Generator.TYPE_MAPPING,
    247            exp.DataType.Type.INT: "INTEGER",
    248            exp.DataType.Type.FLOAT: "REAL",
    249            exp.DataType.Type.BINARY: "VARBINARY",
    250            exp.DataType.Type.TEXT: "VARCHAR",
    251            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    252            exp.DataType.Type.STRUCT: "ROW",
    253        }
    254
    255        TRANSFORMS = {
    256            **generator.Generator.TRANSFORMS,
    257            exp.ApproxDistinct: _approx_distinct_sql,
    258            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    259            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    260            exp.ArrayConcat: rename_func("CONCAT"),
    261            exp.ArrayContains: rename_func("CONTAINS"),
    262            exp.ArraySize: rename_func("CARDINALITY"),
    263            exp.BitwiseAnd: lambda self, e: f"BITWISE_AND({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    264            exp.BitwiseLeftShift: lambda self, e: f"BITWISE_ARITHMETIC_SHIFT_LEFT({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    265            exp.BitwiseNot: lambda self, e: f"BITWISE_NOT({self.sql(e, 'this')})",
    266            exp.BitwiseOr: lambda self, e: f"BITWISE_OR({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    267            exp.BitwiseRightShift: lambda self, e: f"BITWISE_ARITHMETIC_SHIFT_RIGHT({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    268            exp.BitwiseXor: lambda self, e: f"BITWISE_XOR({self.sql(e, 'this')}, {self.sql(e, 'expression')})",
    269            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    270            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    271            exp.DataType: _datatype_sql,
    272            exp.DateAdd: lambda self, e: self.func(
    273                "DATE_ADD", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
    274            ),
    275            exp.DateDiff: lambda self, e: self.func(
    276                "DATE_DIFF", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
    277            ),
    278            exp.DateStrToDate: lambda self, e: f"CAST(DATE_PARSE({self.sql(e, 'this')}, {Presto.DATE_FORMAT}) AS DATE)",
    279            exp.DateToDi: lambda self, e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    280            exp.Decode: _decode_sql,
    281            exp.DiToDate: lambda self, e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    282            exp.Encode: _encode_sql,
    283            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    284            exp.Group: transforms.preprocess([transforms.unalias_group]),
    285            exp.Hex: rename_func("TO_HEX"),
    286            exp.If: if_sql,
    287            exp.ILike: no_ilike_sql,
    288            exp.Initcap: _initcap_sql,
    289            exp.Lateral: _explode_to_unnest_sql,
    290            exp.Left: left_to_substring_sql,
    291            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    292            exp.LogicalAnd: rename_func("BOOL_AND"),
    293            exp.LogicalOr: rename_func("BOOL_OR"),
    294            exp.Pivot: no_pivot_sql,
    295            exp.Quantile: _quantile_sql,
    296            exp.Right: right_to_substring_sql,
    297            exp.SafeBracket: lambda self, e: self.func(
    298                "ELEMENT_AT", e.this, seq_get(apply_index_offset(e.this, e.expressions, 1), 0)
    299            ),
    300            exp.SafeDivide: no_safe_divide_sql,
    301            exp.Schema: _schema_sql,
    302            exp.Select: transforms.preprocess(
    303                [
    304                    transforms.eliminate_qualify,
    305                    transforms.eliminate_distinct_on,
    306                    transforms.explode_to_unnest,
    307                ]
    308            ),
    309            exp.SortArray: _no_sort_array,
    310            exp.StrPosition: rename_func("STRPOS"),
    311            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    312            exp.StrToTime: _str_to_time_sql,
    313            exp.StrToUnix: lambda self, e: f"TO_UNIXTIME(DATE_PARSE({self.sql(e, 'this')}, {self.format_time(e)}))",
    314            exp.StructExtract: struct_extract_sql,
    315            exp.Table: transforms.preprocess([_unnest_sequence]),
    316            exp.TimestampTrunc: timestamptrunc_sql,
    317            exp.TimeStrToDate: timestrtotime_sql,
    318            exp.TimeStrToTime: timestrtotime_sql,
    319            exp.TimeStrToUnix: lambda self, e: f"TO_UNIXTIME(DATE_PARSE({self.sql(e, 'this')}, {Presto.TIME_FORMAT}))",
    320            exp.TimeToStr: lambda self, e: f"DATE_FORMAT({self.sql(e, 'this')}, {self.format_time(e)})",
    321            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    322            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    323            exp.TsOrDiToDi: lambda self, e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    324            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    325            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    326            exp.Unhex: rename_func("FROM_HEX"),
    327            exp.UnixToStr: lambda self, e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    328            exp.UnixToTime: rename_func("FROM_UNIXTIME"),
    329            exp.UnixToTimeStr: lambda self, e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    330            exp.VariancePop: rename_func("VAR_POP"),
    331            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    332            exp.WithinGroup: transforms.preprocess(
    333                [transforms.remove_within_group_for_percentiles]
    334            ),
    335        }
    336
    337        def interval_sql(self, expression: exp.Interval) -> str:
    338            unit = self.sql(expression, "unit")
    339            if expression.this and unit.lower().startswith("week"):
    340                return f"({expression.this.name} * INTERVAL '7' day)"
    341            return super().interval_sql(expression)
    342
    343        def transaction_sql(self, expression: exp.Transaction) -> str:
    344            modes = expression.args.get("modes")
    345            modes = f" {', '.join(modes)}" if modes else ""
    346            return f"START TRANSACTION{modes}"
    347
    348        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    349            start = expression.args["start"]
    350            end = expression.args["end"]
    351            step = expression.args.get("step")
    352
    353            if isinstance(start, exp.Cast):
    354                target_type = start.to
    355            elif isinstance(end, exp.Cast):
    356                target_type = end.to
    357            else:
    358                target_type = None
    359
    360            if target_type and target_type.is_type("timestamp"):
    361                to = target_type.copy()
    362
    363                if target_type is start.to:
    364                    end = exp.cast(end, to)
    365                else:
    366                    start = exp.cast(start, to)
    367
    368            return self.func("SEQUENCE", start, end, step)
    369
    370        def offset_limit_modifiers(
    371            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    372        ) -> t.List[str]:
    373            return [
    374                self.sql(expression, "offset"),
    375                self.sql(limit),
    376            ]

Generator converts a given syntax tree to the corresponding SQL string.

###### Arguments:

- pretty: Whether or not to format the produced SQL string. Default: False.
- identify: Determines when an identifier should be quoted. Possible values are: False (default): Never quote, except in cases where it's mandatory by the dialect. True or 'always': Always quote. 'safe': Only quote identifiers that are case insensitive.
- normalize: Whether or not to normalize identifiers to lowercase. Default: False.
- pad: Determines the pad size in a formatted string. Default: 2.
- indent: Determines the indentation size in a formatted string. Default: 2.
- normalize_functions: Whether or not to normalize all function names. Possible values are: "upper" or True (default): Convert names to uppercase. "lower": Convert names to lowercase. False: Disables function name normalization.
- unsupported_level: Determines the generator's behavior when it encounters unsupported expressions. Default ErrorLevel.WARN.
- max_unsupported: Maximum number of unsupported messages to include in a raised UnsupportedError. This is only relevant if unsupported_level is ErrorLevel.RAISE. Default: 3
- leading_comma: Determines whether or not the comma is leading or trailing in select expressions. This is only relevant when generating in pretty mode. Default: False
- max_text_width: The max number of characters in a segment before creating new lines in pretty mode. The default is on the smaller end because the length only represents a segment and not the true line length. Default: 80
- comments: Whether or not to preserve comments in the output SQL code. Default: True

INTERVAL_ALLOWS_PLURAL_FORM = False
JOIN_HINTS = False
TABLE_HINTS = False
IS_BOOL_ALLOWED = False
STRUCT_DELIMITER = ('(', ')')
PROPERTIES_LOCATION = {<class 'sqlglot.expressions.AlgorithmProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.AutoIncrementProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BlockCompressionProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CharacterSetProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ChecksumProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CollateProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Cluster'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ClusteredByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DataBlocksizeProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.DefinerProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DictRange'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistStyleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.EngineProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExternalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.FallbackProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.FileFormatProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.FreespaceProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.IsolatedLoadingProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.JournalProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.LanguageProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LikeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LocationProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.LockingProperty'>: <Location.POST_ALIAS: 'POST_ALIAS'>, <class 'sqlglot.expressions.LogProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.MaterializedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeBlockRatioProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.OnCommitProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.Order'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedByProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.PrimaryKey'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Property'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.ReturnsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatDelimitedProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatSerdeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SerdeProperties'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Set'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SettingsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SetProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SortKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.StabilityProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TemporaryProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ToTableProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TransientProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeTreeTTL'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.VolatileProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.WithDataProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <Location.POST_NAME: 'POST_NAME'>}
TYPE_MAPPING = {<Type.NCHAR: 'NCHAR'>: 'CHAR', <Type.NVARCHAR: 'NVARCHAR'>: 'VARCHAR', <Type.MEDIUMTEXT: 'MEDIUMTEXT'>: 'TEXT', <Type.LONGTEXT: 'LONGTEXT'>: 'TEXT', <Type.MEDIUMBLOB: 'MEDIUMBLOB'>: 'BLOB', <Type.LONGBLOB: 'LONGBLOB'>: 'BLOB', <Type.INET: 'INET'>: 'INET', <Type.INT: 'INT'>: 'INTEGER', <Type.FLOAT: 'FLOAT'>: 'REAL', <Type.BINARY: 'BINARY'>: 'VARBINARY', <Type.TEXT: 'TEXT'>: 'VARCHAR', <Type.TIMESTAMPTZ: 'TIMESTAMPTZ'>: 'TIMESTAMP', <Type.STRUCT: 'STRUCT'>: 'ROW'}
TRANSFORMS = {<class 'sqlglot.expressions.DateAdd'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TsOrDsAdd'>: <function _ts_or_ds_add_sql>, <class 'sqlglot.expressions.CaseSpecificColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CheckColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CollateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CommentColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DateFormatColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DefaultColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EncodeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExternalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InlineLengthColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LanguageProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LocationProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LogProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.MaterializedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnCommitProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnUpdateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.PathColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ReturnsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SettingsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StabilityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TemporaryProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransientProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TitleColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UppercaseColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VolatileProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ApproxDistinct'>: <function _approx_distinct_sql>, <class 'sqlglot.expressions.ApproxQuantile'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Array'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayConcat'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayContains'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArraySize'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.BitwiseAnd'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseLeftShift'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseNot'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseOr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseRightShift'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseXor'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Cast'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.CurrentTimestamp'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DataType'>: <function _datatype_sql>, <class 'sqlglot.expressions.DateDiff'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateStrToDate'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateToDi'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Decode'>: <function _decode_sql>, <class 'sqlglot.expressions.DiToDate'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Encode'>: <function _encode_sql>, <class 'sqlglot.expressions.FileFormatProperty'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Group'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.Hex'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.If'>: <function if_sql>, <class 'sqlglot.expressions.ILike'>: <function no_ilike_sql>, <class 'sqlglot.expressions.Initcap'>: <function _initcap_sql>, <class 'sqlglot.expressions.Lateral'>: <function _explode_to_unnest_sql>, <class 'sqlglot.expressions.Left'>: <function left_to_substring_sql>, <class 'sqlglot.expressions.Levenshtein'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalAnd'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalOr'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Pivot'>: <function no_pivot_sql>, <class 'sqlglot.expressions.Quantile'>: <function _quantile_sql>, <class 'sqlglot.expressions.Right'>: <function right_to_substring_sql>, <class 'sqlglot.expressions.SafeBracket'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.SafeDivide'>: <function no_safe_divide_sql>, <class 'sqlglot.expressions.Schema'>: <function _schema_sql>, <class 'sqlglot.expressions.Select'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.SortArray'>: <function _no_sort_array>, <class 'sqlglot.expressions.StrPosition'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.StrToDate'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.StrToTime'>: <function _str_to_time_sql>, <class 'sqlglot.expressions.StrToUnix'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.StructExtract'>: <function struct_extract_sql>, <class 'sqlglot.expressions.Table'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.TimestampTrunc'>: <function timestamptrunc_sql>, <class 'sqlglot.expressions.TimeStrToDate'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeStrToTime'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeStrToUnix'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToUnix'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.TryCast'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.TsOrDiToDi'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TsOrDsToDate'>: <function _ts_or_ds_to_date_sql>, <class 'sqlglot.expressions.Unhex'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.UnixToStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.UnixToTime'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.UnixToTimeStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.VariancePop'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.With'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.WithinGroup'>: <function preprocess.<locals>._to_sql>}
def interval_sql(self, expression: sqlglot.expressions.Interval) -> str: View Source

    337        def interval_sql(self, expression: exp.Interval) -> str:
    338            unit = self.sql(expression, "unit")
    339            if expression.this and unit.lower().startswith("week"):
    340                return f"({expression.this.name} * INTERVAL '7' day)"
    341            return super().interval_sql(expression)

def transaction_sql(self, expression: sqlglot.expressions.Transaction) -> str: View Source

    343        def transaction_sql(self, expression: exp.Transaction) -> str:
    344            modes = expression.args.get("modes")
    345            modes = f" {', '.join(modes)}" if modes else ""
    346            return f"START TRANSACTION{modes}"

def generateseries_sql(self, expression: sqlglot.expressions.GenerateSeries) -> str: View Source

    348        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    349            start = expression.args["start"]
    350            end = expression.args["end"]
    351            step = expression.args.get("step")
    352
    353            if isinstance(start, exp.Cast):
    354                target_type = start.to
    355            elif isinstance(end, exp.Cast):
    356                target_type = end.to
    357            else:
    358                target_type = None
    359
    360            if target_type and target_type.is_type("timestamp"):
    361                to = target_type.copy()
    362
    363                if target_type is start.to:
    364                    end = exp.cast(end, to)
    365                else:
    366                    start = exp.cast(start, to)
    367
    368            return self.func("SEQUENCE", start, end, step)

def offset_limit_modifiers( self, expression: sqlglot.expressions.Expression, fetch: bool, limit: Union[sqlglot.expressions.Fetch, sqlglot.expressions.Limit, NoneType]) -> List[str]: View Source

    370        def offset_limit_modifiers(
    371            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    372        ) -> t.List[str]:
    373            return [
    374                self.sql(expression, "offset"),
    375                self.sql(limit),
    376            ]

SELECT_KINDS: Tuple[str, ...] = ()
INVERSE_TIME_MAPPING: Dict[str, str] = {'%B': '%M', '%-m': '%c', '%-d': '%e', '%I': '%h', '%M': '%i', '%S': '%S', '%W': '%u', '%-H': '%k', '%-I': '%l', '%H:%M:%S': '%T', '%a': '%W'}
INVERSE_TIME_TRIE: Dict = {'%': {'B': {0: True}, '-': {'m': {0: True}, 'd': {0: True}, 'H': {0: True}, 'I': {0: True}}, 'I': {0: True}, 'M': {0: True}, 'S': {0: True}, 'W': {0: True}, 'H': {':': {'%': {'M': {':': {'%': {'S': {0: True}}}}}}}, 'a': {0: True}}}
INDEX_OFFSET = 1
STRICT_STRING_CONCAT = True
NULL_ORDERING = 'nulls_are_last'
@classmethod
def can_identify(text: str, identify: str | bool = 'safe') -> bool: View Source

    247    @classmethod
    248    def can_identify(cls, text: str, identify: str | bool = "safe") -> bool:
    249        """Checks if text can be identified given an identify option.
    250
    251        Args:
    252            text: The text to check.
    253            identify:
    254                "always" or `True`: Always returns true.
    255                "safe": True if the identifier is case-insensitive.
    256
    257        Returns:
    258            Whether or not the given text can be identified.
    259        """
    260        if identify is True or identify == "always":
    261            return True
    262
    263        if identify == "safe":
    264            return not cls.case_sensitive(text)
    265
    266        return False

Checks if text can be identified given an identify option.

###### Arguments:

- text: The text to check.
- identify: "always" or `True`: Always returns true. "safe": True if the identifier is case-insensitive.

###### Returns:

> Whether or not the given text can be identified.

QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
STRING_ESCAPE = "'"
IDENTIFIER_ESCAPE = '"'
BIT_START: Optional[str] = None
BIT_END: Optional[str] = None
HEX_START: Optional[str] = None
HEX_END: Optional[str] = None
BYTE_START: Optional[str] = None
BYTE_END: Optional[str] = None
RAW_START: Optional[str] = None
RAW_END: Optional[str] = None

##### Inherited Members

sqlglot.generator.Generator
  Generator
  NULL_ORDERING_SUPPORTED
  LOCKING_READS_SUPPORTED
  EXPLICIT_UNION
  WRAP_DERIVED_VALUES
  CREATE_FUNCTION_RETURN_AS
  MATCHED_BY_SOURCE
  SINGLE_STRING_INTERVAL
  TABLESAMPLE_WITH_METHOD
  TABLESAMPLE_SIZE_IS_PERCENT
  LIMIT_FETCH
  RENAME_TABLE_WITH_DB
  GROUPINGS_SEP
  INDEX_ON
  STAR_MAPPING
  TIME_PART_SINGULARS
  TOKEN_MAPPING
  PARAMETER_TOKEN
  RESERVED_KEYWORDS
  WITH_SEPARATED_COMMENTS
  UNWRAPPED_INTERVAL_VALUES
  SENTINEL_LINE_BREAK
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  NORMALIZE_FUNCTIONS
  pretty
  identify
  normalize
  pad
  unsupported_level
  max_unsupported
  leading_comma
  max_text_width
  comments
  normalize_functions
  unsupported_messages
  generate
  unsupported
  sep
  seg
  pad_comment
  maybe_comment
  wrap
  no_identify
  normalize_func
  indent
  sql
  uncache_sql
  cache_sql
  characterset_sql
  column_sql
  columnposition_sql
  columndef_sql
  columnconstraint_sql
  autoincrementcolumnconstraint_sql
  compresscolumnconstraint_sql
  generatedasidentitycolumnconstraint_sql
  notnullcolumnconstraint_sql
  primarykeycolumnconstraint_sql
  uniquecolumnconstraint_sql
  createable_sql
  create_sql
  clone_sql
  describe_sql
  prepend_ctes
  with_sql
  cte_sql
  tablealias_sql
  bitstring_sql
  hexstring_sql
  bytestring_sql
  rawstring_sql
  datatypesize_sql
  datatype_sql
  directory_sql
  delete_sql
  drop_sql
  except_sql
  except_op
  fetch_sql
  filter_sql
  hint_sql
  index_sql
  identifier_sql
  inputoutputformat_sql
  national_sql
  partition_sql
  properties_sql
  root_properties
  properties
  with_properties
  locate_properties
  property_sql
  likeproperty_sql
  fallbackproperty_sql
  journalproperty_sql
  freespaceproperty_sql
  checksumproperty_sql
  mergeblockratioproperty_sql
  datablocksizeproperty_sql
  blockcompressionproperty_sql
  isolatedloadingproperty_sql
  lockingproperty_sql
  withdataproperty_sql
  insert_sql
  intersect_sql
  intersect_op
  introducer_sql
  pseudotype_sql
  onconflict_sql
  returning_sql
  rowformatdelimitedproperty_sql
  withtablehint_sql
  indextablehint_sql
  table_sql
  tablesample_sql
  pivot_sql
  tuple_sql
  update_sql
  values_sql
  var_sql
  into_sql
  from_sql
  group_sql
  having_sql
  join_sql
  lambda_sql
  lateral_sql
  limit_sql
  offset_sql
  setitem_sql
  set_sql
  pragma_sql
  lock_sql
  literal_sql
  escape_str
  loaddata_sql
  null_sql
  boolean_sql
  order_sql
  cluster_sql
  distribute_sql
  sort_sql
  ordered_sql
  matchrecognize_sql
  query_modifiers
  after_having_modifiers
  after_limit_modifiers
  select_sql
  schema_sql
  schema_columns_sql
  star_sql
  parameter_sql
  sessionparameter_sql
  placeholder_sql
  subquery_sql
  qualify_sql
  union_sql
  union_op
  unnest_sql
  where_sql
  window_sql
  partition_by_sql
  windowspec_sql
  withingroup_sql
  between_sql
  bracket_sql
  safebracket_sql
  all_sql
  any_sql
  exists_sql
  case_sql
  constraint_sql
  nextvaluefor_sql
  extract_sql
  trim_sql
  safeconcat_sql
  check_sql
  foreignkey_sql
  primarykey_sql
  if_sql
  matchagainst_sql
  jsonkeyvalue_sql
  jsonobject_sql
  openjsoncolumndef_sql
  openjson_sql
  in_sql
  in_unnest_op
  return_sql
  reference_sql
  anonymous_sql
  paren_sql
  neg_sql
  not_sql
  alias_sql
  aliases_sql
  attimezone_sql
  add_sql
  and_sql
  connector_sql
  bitwiseand_sql
  bitwiseleftshift_sql
  bitwisenot_sql
  bitwiseor_sql
  bitwiserightshift_sql
  bitwisexor_sql
  cast_sql
  currentdate_sql
  collate_sql
  command_sql
  comment_sql
  mergetreettlaction_sql
  mergetreettl_sql
  commit_sql
  rollback_sql
  altercolumn_sql
  renametable_sql
  altertable_sql
  droppartition_sql
  addconstraint_sql
  distinct_sql
  ignorenulls_sql
  respectnulls_sql
  intdiv_sql
  dpipe_sql
  safedpipe_sql
  div_sql
  overlaps_sql
  distance_sql
  dot_sql
  eq_sql
  escape_sql
  glob_sql
  gt_sql
  gte_sql
  ilike_sql
  ilikeany_sql
  is_sql
  like_sql
  likeany_sql
  similarto_sql
  lt_sql
  lte_sql
  mod_sql
  mul_sql
  neq_sql
  nullsafeeq_sql
  nullsafeneq_sql
  or_sql
  slice_sql
  sub_sql
  trycast_sql
  use_sql
  binary
  function_fallback_sql
  func
  format_args
  text_width
  format_time
  expressions
  op_expressions
  naked_property
  set_operation
  tag_sql
  token_sql
  userdefinedfunction_sql
  joinhint_sql
  kwarg_sql
  when_sql
  merge_sql
  tochar_sql
  dictproperty_sql
  dictrange_sql
  dictsubproperty_sql
  oncluster_sql
  clusteredbyproperty_sql